/*
 * Misc GP2X related stuff.
 * (C) 2006 Väinö Helminen
 * $Id: gp2x.c,v 1.1.1.1 2006-12-11 04:27:11 vah Exp $
 */
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>

#include "gp2x.h"

static volatile unsigned long *gp2x_memregl = 0;
static volatile unsigned short *gp2x_memregs = 0;

static void GP2XQuit(void)
{
  if(gp2x_memregl) munmap((void*)gp2x_memregl, 0x10000);
  chdir("/usr/gp2x");
  execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
}

void GP2XInit(void)
{
  int fd;

  if(atexit(GP2XQuit))
    GP2XQuit(); /* Function registration failed. Exit now, so that GP2X doesn't "hang". */

  fd = open("/dev/mem", O_RDWR);
  gp2x_memregl=mmap(0, 0x10000, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0xc0000000);
  gp2x_memregs=(void*)gp2x_memregl;
  close(fd);
}

void GP2XWaitVBlank(void)
{
  while(gp2x_memregs[0x1182>>1]&(1<<4));
  while(!(gp2x_memregs[0x1182>>1]&(1<<4)));
}

void GP2XCpuFrequency(unsigned int mhz)
{
  unsigned int m, l;

  /* Calculating the FPLL value from frequency is a bit complex
   * so we just use some known-to-work values instead. */
   switch(mhz) {
   case 250: m = 0x5D04; break;
   case 225: m = 0x5304; break;
   case 200: m = 0x4904; break;
   case 175: m = 0x3F04; break;
   case 150: m = 0x4901; break;
   case 125: m = 0x3c01; break;
   case 100: m = 0x6502; break;
   case 75: m = 0x4902; break;
   default:
   case 50: m = 0x6503; break;
   }
  
   l = gp2x_memregl[0x808>>2];          /* get interupt flags */
   gp2x_memregl[0x808>>2] = 0xFF8FFFE7; /* turn off interrupts */
   gp2x_memregs[0x910>>1] = m;          /* set new clock frequency */
   while(gp2x_memregs[0x0902>>1] & 1);  /* wait for it to take */
   gp2x_memregl[0x808>>2] = l;          /* turn on interrupts */
}
